@@ -101,7 +101,7 @@ trans_10pow <- function(x) {
 #' @title 10 power x transformation
 #' @description
 #' Parameter values x are transformed to \code{1-10^x}.
-#' This is helpful for parameters that are likely to be set very close to (but below) a value of 1, 
+#' This is helpful for parameters that are likely to be set very close to (but below) a value of 1,
 #' such as discount factors in reinforcement learning.
 #'
 #' @param x input
@@ -201,7 +201,7 @@ getBudget <- function(runtime_bench,
 }
 
 
-#' @title Get objective function
+#' @title Get objective function for mlr
 #'
 #' @description mlrTools
 #' This function receives a configuration for a tuning experiment,
@@ -210,7 +210,11 @@ getBudget <- function(runtime_bench,
 #' @param config list
 #' @param timeout integer, time in seconds after which a model (learner) evaluation will be aborted.
 #'
-#' @return an objective function that can be optimized via \code{\link[SPOT]{spot}}
+#' @return an objective function that can be optimized via \code{\link[SPOT]{spot}}.
+#' It basically provides the result from a call to \code{\link[mlr]{resample}}:
+#' \code{resample(lrn, task, resample, measures = measures, show.info = FALSE)},
+#' with measures defined as \code{rmse}, \code{timeboth}, \code{timetrain}, and
+#' \code{timepredict}.
 #'
 #' @export
 #' @importFrom callr r
@@ -813,5 +817,255 @@ prepare_spot_result_plot <- function(data,model=buildRanger,modelControl=list(),
 }
 
 
+#' @title Get experiment configuration
+#'
+#' @desription Based on an mlr task, this function generates an experiment configuration for
+#' tuning a specified model.
+#'
+#' @param task.type character, either "classif" or "regr".
+#' @param model character, learner chosen via mlr (here: "rpart", kknn", "cvglmnet","ranger","xgboost","svm).
+#' @param rsmpl resample strategy, e.g., generated by \code{makeFixedHoldoutInstance}.
+#' @param task list, e.g., generated by \code{\link{get_task_census}}.
+#' @param data.seed this integer will be used to set the RNG before sampling the test and training data from the complete dataset.
+#'
+#' @return an mlr task with the respective data set
+#'
+#' @export
+#' @import mlr
+get_exp_config <- function(task.type,model,task,rsmpl,data.seed){
+
+  ## specify learner rpart
+  if(model=="rpart"){
+    learner <- paste(task.type,"rpart",sep=".")
+    #pars <- getParamSet(learner)
+    #pars
+    tunepars <- c("minsplit","minbucket","cp","maxdepth")
+    lower <- c(1, ## Prob19a
+               0.1, ## Prob19a
+               -10, ## slightly larger than Prob19a who allow -Inf, i.e., zero on actual scale
+               1) ## Prob19a
+    upper <- c(300, ## larger than Prob19a (60)
+               0.5, ## a bit larger than Prob19a  (60)
+               0, ##Prob19a
+               30) ## Prob19a, maximum in rpart
+    type <-  c("integer","numeric","numeric","integer")
+    fixpars <- list()
+    factorlevels <- list()
+    transformations <- c(trans_id,
+                         trans_id,
+                         trans_10pow, ##prob19a use trans_id for cp.
+                         trans_id)
+    dummy=FALSE
+    relpars <- list(minbucket=expression(round(max(minsplit*minbucket,1))))
+  }
+
+  ## specify learner kknn
+  if(model=="kknn"){
+    learner <- paste(task.type,"kknn",sep=".")
+    #pars <- getParamSet(learner)
+    #pars
+    tunepars <- c("k","distance")
+    lower <- c(1, ## Prob19a
+               -1) ##
+    upper <- c(30,  ## Prob19a
+               2) ##
+    type <-  c("integer","numeric")
+    fixpars <- list()
+    factorlevels <- list()
+    transformations <- c(trans_id,
+                         trans_10pow) ##?
+    dummy=FALSE
+    relpars <- list()
+  }
+
+  ## specify learner EN
+  if(model=="cvglmnet"){
+    learner <- paste(task.type,"cvglmnet",sep=".")
+    #pars <- getParamSet(learner)
+    #pars
+    tunepars <- c("alpha","thresh")
+    lower <- c(0, ## Prob19a
+               -8) ## 3 magnitudes below default
+    upper <- c(1, ## Prob19a
+               -1) ## ?
+    type <-  c("numeric","numeric")
+    fixpars <- list()
+    factorlevels <- list()
+    transformations <- c(trans_id, ## Prob19a
+                         trans_10pow) ## this assigns more relevance to lower values.
+    dummy=FALSE
+    relpars <- list()
+  }
+
+  ## specify learner RF
+  if(model=="ranger"){
+    learner <- paste(task.type,"ranger",sep=".")
+    #pars <- getParamSet(learner)
+    #pars
+    tunepars <- c("num.trees","mtry","sample.fraction","replace","respect.unordered.factors")
+    lower <- c(0, ## => 1, Prob19a
+               1, ## minimum, Prob19a
+               0.1, ## Prob19a
+               1, ## => FALSE
+               1) ## => ignore
+    upper <- c(11, ## => 2048 (very similar to Prob19a)
+               getTaskNFeats(task), ## Prob19a but on original scale.
+               1, ## Prob19a
+               2, ## => TRUE
+               2) ## => order
+    type <-  c("numeric","integer","numeric","factor","factor")
+    fixpars <- list(num.threads=1)
+    factorlevels <- list(respect.unordered.factors=c("ignore","order","partition"),
+                         ## Note: partition not used. extreme run time
+                         replace=c(FALSE,TRUE)
+    )
+    transformations <- c(trans_2pow_round, ## deviates from Prob19a.
+                         ## Reasoning: increasing number of trees leads to decreasing change per unit.
+                         trans_id,
+                         trans_id,
+                         trans_id,
+                         trans_id)
+    dummy=FALSE
+    relpars <- list()
+  }
+
+  ## specify learner svm
+  if(model=="svm"){
+    learner <- paste(task.type,"svm",sep=".")
+    if(task.type=="classif"){
+      #pars <- getParamSet(learner)
+      #pars
+      tunepars <- c("kernel","cost","gamma","coef0")
+      #"degree" only with poly kernel
+      lower <-    c(1,   ## radial
+                    -10, ## Prob19a
+                    -10, ## Prob19a
+                    -1)  ## Rijn18a
+      upper <-    c(2,  ## sigmoid
+                    10, ## Prob19a
+                    10, ## Prob19a
+                    1)  ## Rijn18a
+      type <-  c("factor","numeric","numeric","numeric")
+      #factorlevels <- list(kernel= c("linear","polynomial","radial","sigmoid"))
+      ## reduce to sigmoid and radial, due to shorter runtimes.
+      factorlevels <- list(kernel= c("radial","sigmoid"))
+      transformations <- c(trans_id,
+                           trans_2pow, ## Prob19a
+                           trans_2pow, ## Prob19a
+                           trans_id)
+    }else if(task.type=="regr"){
+      #pars <- getParamSet(learner)
+      #pars
+      tunepars <- c("kernel","cost","gamma","coef0","epsilon")
+      lower <-    c(1,   ## radial
+                    -10, ## Prob19a
+                    -10, ## Prob19a
+                    -1,  ## Rijn18a
+                    -8)
+      #upper <-    c(4, 10,  10,  5, 1)
+      upper <-    c(2,  ## sigmoid
+                    10, ## Prob19a
+                    10, ## Prob19a
+                    1,  ## Rijn18a
+                    0)
+      type <-  c("factor","numeric","numeric","numeric","numeric")
+      #factorlevels <- list(kernel= c("linear","polynomial","radial","sigmoid"))
+      ## reduce to sigmoid and radial, due to shorter runtimes.
+      factorlevels <- list(kernel= c("radial","sigmoid"))
+      transformations <- c(trans_id,
+                           trans_2pow, ## Prob19a
+                           trans_2pow, ## Prob19a
+                           trans_id,
+                           trans_10pow
+      )
+    }
+    fixpars <- list()
+    dummy=FALSE
+    relpars <- list()
+  }
+
+  ## specify learner xgb
+  if(model=="xgboost"){
+    learner <- paste(task.type,"xgboost",sep=".")
+    #pars <- getParamSet(learner)
+    #pars
+    tunepars <- c("nrounds","eta","lambda","alpha","subsample","colsample_bytree","gamma","max_depth","min_child_weight")
+    lower <- c(0,  ## reasonable minimum.
+               -10, ## Prob19a
+               -10, ## Prob19a
+               -10, ## Prob19a
+               0.1, ## Prob19a
+               1/getTaskNFeats(task),
+               ## slight deviation from Prob19a.
+               ## reason: 0 makes no sense. At least one feature should be chosen via colsample.
+               -10, ## etwas kleiner als Thom18a
+               1,   ## Prob19a
+               0)   ## Prob19a
+    upper <- c(11,  ## set similar as random forest. (which is less than Prob19a used: 5000)
+               0,  ## Prob19a
+               10,  ## Prob19a
+               10,  ## Prob19a
+               1,   ## Prob19a
+               1,   ## Prob19a
+               10,  ## etwas größer als Thom18a
+               15,  ## Prob19a
+               7) ## Prob19a
+    type <-  c("numeric","numeric","numeric","numeric","numeric","numeric","numeric","integer","numeric")
+    if(task.type=="classif"){
+      fixpars <- list(eval_metric="logloss",# suppress warning given when default metric is used.
+                      nthread=1) #one thread, not parallel
+    }else{
+      fixpars <- list(eval_metric="rmse",# suppress warning given when default metric is used.
+                      nthread=1) #one thread, not parallel
+    }
+    factorlevels <- list()
+    transformations <- c(trans_2pow_round, ## differs from Prob19a
+                         trans_2pow, ## Prob19a
+                         trans_2pow, ## Prob19a
+                         trans_2pow, ## Prob19a
+                         trans_id,   ## Prob19a
+                         trans_id,   ## Prob19a
+                         trans_2pow, ## Thom18a
+                         trans_id,   ## Prob19a
+                         trans_2pow) ## Prob19a
+    dummy=TRUE
+    ## TODO/Note: number of features will be changed. consider when setting bounds. problem for colsample?
+    relpars <- list()
+  }
+  ## Impute missing values.
+  task <- impute(
+    task,
+    classes = list(
+      factor = imputeMode(),
+      integer = imputeMedian(),
+      numeric = imputeMean()
+    )
+  )$task
+  if(dummy){
+    task <- createDummyFeatures(task)
+  }
+  ## Determine test/train split
+  set.seed(data.seed)
+  train.id <- sample(1:getTaskSize(task),size = getTaskSize(task)*0.6,replace=FALSE)
+  test.id <- (1:getTaskSize(task))[-train.id]
+  return(list(
+    learner=learner,
+    tunepars=tunepars,
+    lower=lower,
+    upper=upper,
+    type=type,
+    fixpars=fixpars,
+    factorlevels=factorlevels,
+    transformations=transformations,
+    dummy=dummy,
+    relpars=relpars,
+    task=task,
+    resample = rsmpl
+  ))
+}
+
+
+
+
 
 
